import asyncio
import json
import logging
from urllib.parse import urljoin

import config
from aiohttp import ClientResponse, ClientSession
from gestion_sports.payload_builders import (
    GestionSportsBookingPayloadBuilder,
    GestionSportsLoginPayloadBuilder,
)
from models import BookingFilter, Club, User

LOGGER = logging.getLogger(__name__)
POST_HEADERS = config.get_post_headers("gestion-sports")


class GestionSportsConnector:
    """
    Handle the specific booking requests to Gestion-Sports
    """

    def __init__(self, session: ClientSession, url: str):
        LOGGER.info("Initializing connection to GestionSports API")
        self.url = url
        self.session = session

    @property
    def landing_url(self) -> str:
        """
        Get the URL to the landing page of Gestion-Sports

        :return: the URL to the landing page
        """
        return urljoin(self.url, "/connexion.php?")

    @property
    def login_url(self) -> str:
        """
        Get the URL to the connection login of Gestion-Sports

        :return: the URL to the login page
        """
        return urljoin(self.url, "/connexion.php?")

    @property
    def booking_url(self) -> str:
        """
        Get the URL to the booking page of Gestion-Sports

        :return: the URL to the booking page
        """
        return urljoin(self.url, "/membre/reservation.html?")

    async def land(self) -> ClientResponse:
        """
        Perform the request to the landing page in order to get the cookie PHPSESSIONID

        :return: the response from the landing page
        """
        LOGGER.info("Connecting to GestionSports API at %s", self.login_url)
        async with self.session.get(self.landing_url) as response:
            await response.text()
            return response

    async def login(self, user: User, club: Club) -> ClientResponse:
        """
        Perform the request to the log in the user

        :return: the response from the login
        """
        LOGGER.info("Logging in to GestionSports API at %s", self.login_url)
        payload_builder = GestionSportsLoginPayloadBuilder()
        payload = payload_builder.user(user).club(club).build()

        async with self.session.post(
            self.login_url, data=payload, headers=POST_HEADERS
        ) as response:
            resp_text = await response.text()
            LOGGER.debug("Connexion request response:\n%s", resp_text)
            return response

    async def book(self, booking_filter: BookingFilter, club: Club) -> int | None:
        """
        Perform a request for each court at the same time to increase the chances to get
        a booking.
        The gestion-sports backend does not allow several bookings at the same time
        so there is no need to make each request one after the other

        :param booking_filter: the booking information
        :param club: the club where to book the court
        :return: the booked court, or None if no court was booked
        """
        LOGGER.info(
            "Booking any available court from GestionSports API at %s", self.booking_url
        )
        # use asyncio to request a booking on every court
        # the gestion-sports backend is able to book only one court for a user
        bookings = await asyncio.gather(
            *[
                self.book_one_court(booking_filter, court_id)
                for court_id in club.courts_ids
            ],
            return_exceptions=True,
        )

        LOGGER.debug("Booking results:\n'%s'", bookings)
        return self.get_booked_court(bookings)

    async def book_one_court(
        self, booking_filter: BookingFilter, court_id: int
    ) -> tuple[int, bool]:
        """
        Book a single court according to the information provided in the booking filter

        :param booking_filter: the booking information
        :param court_id: the id of the court to book
        :return: a tuple containing the court id and the booking status
        """
        LOGGER.debug(
            "Booking court %s at %s",
            court_id,
            booking_filter.date.to_w3c_string(),
        )
        payload_builder = GestionSportsBookingPayloadBuilder()
        payload = (
            payload_builder.booking_filter(booking_filter).court_id(court_id).build()
        )
        LOGGER.debug("Booking court %s request:\n'%s'", court_id, payload)

        async with self.session.post(
            self.booking_url, data=payload, headers=POST_HEADERS
        ) as response:
            resp_json = await response.text()
        LOGGER.debug("Response from booking request:\n'%s'", resp_json)
        return court_id, self.is_response_status_ok(resp_json)

    @staticmethod
    def get_booked_court(bookings):
        for court, is_booked in bookings:
            if is_booked:
                LOGGER.debug("Court %s is booked", court)
                return court
        LOGGER.debug("No booked court found")
        return None

    @staticmethod
    def is_response_status_ok(response: str) -> bool:
        formatted_result = response.removeprefix('"').removesuffix('"')
        result_json = json.loads(formatted_result)
        return result_json["status"] == "ok"

    def get_user_information(self):
        pass
