import logging
import time

import pendulum
from aiohttp import ClientSession
from gestion_sports.gestion_sports_connector import GestionSportsConnector
from models import BookingFilter, Club, User
from pendulum import DateTime

LOGGER = logging.getLogger(__name__)


class GestionSportsPlatform:
    def __init__(self, club: Club):
        LOGGER.info("Initializing Gestion Sports platform at url %s", club.url)
        self.connector: GestionSportsConnector | None = None
        self.club: Club = club
        self.session: ClientSession | None = None

    async def __aenter__(self):
        self.session = ClientSession()
        self.connector = GestionSportsConnector(self.session, self.club.url)
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.session.close()

    async def book(self, user: User, booking_filter: BookingFilter) -> int | None:
        if self.connector is None:
            LOGGER.error("No connection to Gestion Sports is available")
            return

        if user is None or booking_filter is None:
            LOGGER.error("Not enough information available to book a court")
            return

        await self.connector.land()
        await self.connector.login(user, self.club)
        wait_until_booking_time(self.club, booking_filter)
        return await self.connector.book(booking_filter, self.club)

    async def user_can_book(self, user: User, club: Club):
        await self.connector.land()
        await self.connector.login(user, self.club)
        await self.connector.get_user_information()


def wait_until_booking_time(club: Club, booking_filter: BookingFilter):
    """
    Wait until the booking is open.
    The booking filter contains the date and time of the booking.
    The club has the information about when the booking is open for that date.

    :param club: the club where to book a court
    :param booking_filter: the booking information
    """
    LOGGER.info("Waiting for booking time")
    booking_datetime = build_booking_datetime(booking_filter, club)
    now = pendulum.now()
    duration_until_booking = booking_datetime - now
    LOGGER.debug(
        "Time to wait before booking: %s:%s:%s",
        "{:0>2}".format(duration_until_booking.hours),
        "{:0>2}".format(duration_until_booking.minutes),
        "{:0>2}".format(duration_until_booking.seconds),
    )

    while now < booking_datetime:
        time.sleep(1)
        now = pendulum.now()
    LOGGER.info("It's booking time!")


def build_booking_datetime(booking_filter: BookingFilter, club: Club) -> DateTime:
    """
    Build the date and time when the booking is open for a given match date.
    The booking filter contains the date and time of the booking.
    The club has the information about when the booking is open for that date.

    :param booking_filter: the booking information
    :param club: the club where to book a court
    :return: the date and time when the booking is open
    """
    date_to_book = booking_filter.date
    booking_date = date_to_book.subtract(days=club.booking_open_days_before)

    booking_hour = club.booking_opening_time.hour
    booking_minute = club.booking_opening_time.minute

    return booking_date.at(booking_hour, booking_minute)
